@@ -23,11 +23,13 @@ from torch import nn, Tensor
 from torch.optim.swa_utils import SWALR
 
 import pytorch_lightning as pl
+from lightning_lite.utilities.types import _LRScheduler
 from pytorch_lightning.callbacks.callback import Callback
 from pytorch_lightning.strategies import DDPFullyShardedStrategy, DeepSpeedStrategy
+from pytorch_lightning.strategies.fully_sharded_native import DDPFullyShardedNativeStrategy
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from pytorch_lightning.utilities.rank_zero import rank_zero_info, rank_zero_warn
-from pytorch_lightning.utilities.types import _LRScheduler, LRSchedulerConfig
+from pytorch_lightning.utilities.types import LRSchedulerConfig
 
 _AVG_FN = Callable[[Tensor, Tensor, Tensor], Tensor]
 
@@ -121,13 +123,13 @@ class StochasticWeightAveraging(Callback):
         self._avg_fn = avg_fn or self.avg_fn
         self._device = device
         self._model_contains_batch_norm: Optional[bool] = None
-        self._average_model: "pl.LightningModule"
+        self._average_model: Optional["pl.LightningModule"] = None
         self._initialized = False
         self._swa_scheduler: Optional[_LRScheduler] = None
         self._scheduler_state: Optional[Dict] = None
         self._init_n_averaged = 0
         self._latest_update_epoch = -1
-        self.momenta: Optional[Dict[nn.modules.batchnorm._BatchNorm, float]] = None
+        self.momenta: Dict[nn.modules.batchnorm._BatchNorm, Optional[float]] = {}
         self._max_epochs: int
 
     @property
@@ -143,7 +145,10 @@ class StochasticWeightAveraging(Callback):
     def pl_module_contains_batch_norm(pl_module: "pl.LightningModule") -> bool:
         return any(isinstance(module, nn.modules.batchnorm._BatchNorm) for module in pl_module.modules())
 
-    def setup(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", stage: Optional[str] = None) -> None:
+    def setup(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", stage: str) -> None:
+        if isinstance(trainer.strategy, (DDPFullyShardedStrategy, DDPFullyShardedNativeStrategy, DeepSpeedStrategy)):
+            raise MisconfigurationException("SWA does not currently support sharded models.")
+
         # copy the model before moving it to accelerator device.
         with pl_module._prevent_trainer_and_dataloaders_deepcopy():
             self._average_model = deepcopy(pl_module)
@@ -155,9 +160,7 @@ class StochasticWeightAveraging(Callback):
         if len(trainer.lr_scheduler_configs) > 1:
             raise MisconfigurationException("SWA currently not supported for more than 1 `lr_scheduler`.")
 
-        if isinstance(trainer.strategy, (DDPFullyShardedStrategy, DeepSpeedStrategy)):
-            raise MisconfigurationException("SWA does not currently support sharded models.")
-
+        assert trainer.max_epochs is not None
         if isinstance(self._swa_epoch_start, float):
             self._swa_epoch_start = int(trainer.max_epochs * self._swa_epoch_start)
 
@@ -177,6 +180,7 @@ class StochasticWeightAveraging(Callback):
             self._initialized = True
 
             # move average model to request device.
+            assert self._average_model is not None
             self._average_model = self._average_model.to(self._device or pl_module.device)
 
             optimizer = trainer.optimizers[0]
@@ -186,6 +190,7 @@ class StochasticWeightAveraging(Callback):
             for lr, group in zip(self._swa_lrs, optimizer.param_groups):
                 group["initial_lr"] = lr
 
+            assert trainer.max_epochs is not None
             self._swa_scheduler = cast(
                 _LRScheduler,
                 SWALR(
@@ -230,12 +235,14 @@ class StochasticWeightAveraging(Callback):
             trainer.current_epoch > self._latest_update_epoch
         ):
             assert self.n_averaged is not None
+            assert self._average_model is not None
             self.update_parameters(self._average_model, pl_module, self.n_averaged, self._avg_fn)
             self._latest_update_epoch = trainer.current_epoch
 
         # Note: No > here in case the callback is saved with the model and training continues
         if trainer.current_epoch == self.swa_end + 1:
             # Transfer weights from average model to pl_module
+            assert self._average_model is not None
             self.transfer_weights(self._average_model, pl_module)
 
             # Reset BatchNorm for update
@@ -264,6 +271,7 @@ class StochasticWeightAveraging(Callback):
             self.reset_momenta()
         elif trainer.current_epoch - 1 == self.swa_end:
             # Last SWA epoch. Transfer weights from average model to pl_module
+            assert self._average_model is not None
             self.transfer_weights(self._average_model, pl_module)
 
     @staticmethod
@@ -290,15 +298,14 @@ class StochasticWeightAveraging(Callback):
                 dtype=module.running_var.dtype,
             )
             self.momenta[module] = module.momentum
-            module.momentum = float()
+            module.momentum = None  # type: ignore[assignment]
             assert module.num_batches_tracked is not None
             module.num_batches_tracked *= 0
 
     def reset_momenta(self) -> None:
         """Adapted from https://github.com/pytorch/pytorch/blob/v1.7.1/torch/optim/swa_utils.py#L164-L165."""
-        assert self.momenta is not None
         for bn_module in self.momenta:
-            bn_module.momentum = self.momenta[bn_module]
+            bn_module.momentum = self.momenta[bn_module]  # type: ignore[assignment]
 
     @staticmethod
     def update_parameters(
